package com.unleashedsoftware.sdk;

import com.google.inject.*;
import com.unleashedsoftware.sdk.example.*;
import com.unleashedsoftware.sdk.model.*;
import com.unleashedsoftware.sdk.security.*;
import org.junit.*;
import java.util.*;

public class UnleashedApiExamples {
    private final static String apiId = "";
    private final static String apiKey = "";

    private Injector injector;
    private SalesOrderGetById salesOrderGetById;
    private SalesOrderSave salesOrderSave;

    @Before
    public void setUp() {
        this.injector = Guice.createInjector();
        this.salesOrderGetById = injector.getInstance(SalesOrderGetById.class);
        this.salesOrderSave = injector.getInstance(SalesOrderSave.class);
    }

    @Test
    public void givenAnExistingSalesOrderId_whenGet_thenSuccess() throws Exception {
        UnleashedApiCredentials credentials = new UnleashedApiCredentials();
        credentials.privateKey = apiKey;
        credentials.authId = apiId;
        salesOrderGetById.credentials = credentials;
        salesOrderGetById.getById("2aaabbca-710f-403d-8de7-a1b160a33b73");
    }

    @Test
    public void givenANonExistingSalesOrderId_whenGet_thenNotSuccess() throws Exception {
        UnleashedApiCredentials credentials = new UnleashedApiCredentials();
        credentials.privateKey = apiKey;
        credentials.authId = apiId;
        salesOrderGetById.credentials = credentials;
        salesOrderGetById.getById("00000000-27bf-42c4-ac1b-066dcbd46001");
    }

    @Test
    public void givenSalesOrderThatHasMissingDetails_whenPost_notSuccess() throws Exception {
        UnleashedApiCredentials credentials = new UnleashedApiCredentials();
        credentials.privateKey = apiKey;
        credentials.authId = apiId;
        salesOrderSave.credentials = credentials;
        salesOrderSave.save(createInvalidSalesOrder("225366ec-27bf-42c4-ac1b-066dcbd46001"));
    }

    @Test
    public void givenSalesOrderWithInvalidId_whenPost_notSuccess() throws Exception {
        UnleashedApiCredentials credentials = new UnleashedApiCredentials();
        credentials.privateKey = apiKey;
        credentials.authId = apiId;
        salesOrderSave.credentials = credentials;
        salesOrderSave.save(createInvalidSalesOrder("225366ec"));
    }

    @Test
    public void givenANewSalesOrder_whenPost_thenSuccess() throws Exception {
        UnleashedApiCredentials credentials = new UnleashedApiCredentials();
        credentials.privateKey = apiKey;
        credentials.authId = apiId;
        salesOrderSave.credentials = credentials;
        salesOrderSave.save(createValidSalesOrder());
    }

    private ApiSalesOrder createInvalidSalesOrder(String id) {
        ApiSalesOrder salesOrder = new ApiSalesOrder();
        salesOrder.id = id;
        return salesOrder;
    }

    public final static String salesOrderId = UUID.randomUUID().toString();
    public final static String salesOrderNumber = String.format("SO-%s", new Date().getTime()/10);
    public final static String customerId = "c1470594-e3fe-4635-bf18-7ab6a24aada5";
    public final static String customerCode = "ACE001";
    public final static String warehouseId = "e6334112-73ed-402c-a38c-0855f97e9490";
    public final static String warehouseCode = "W1";
    public final static String currencyId = "29e55956-abf5-4997-bd54-c7932f0730b2";
    public final static String currencyCode = "NZD";
    public final static String taxId = "99150966-8e09-4fb2-a11a-6e586c108565";
    public final static String taxCode = "G.S.T.";
    public final static String salesOrderLine1Id = UUID.randomUUID().toString();
    public final static String product1Id = "3e736937-b795-41c1-b79c-ad8414cc8d84";
    public final static String product1Code = "ANIMAL";
    public final static String salesOrderLine2Id = UUID.randomUUID().toString();
    public final static String product2Id = "c479e256-25eb-4abe-9b25-12edcb2a2907";
    public final static String product2Code = "CANDY";

    private ApiSalesOrder createValidSalesOrder() {
        ApiSalesOrder salesOrder = new ApiSalesOrder();
        salesOrder.id = salesOrderId;
        salesOrder.lastModifiedOn = new Date();
        salesOrder.orderNumber = salesOrderNumber;
        salesOrder.orderDate = new Date();
        salesOrder.quoteExpiryDate = new Date();
        salesOrder.requiredDate = new Date();
        salesOrder.orderStatus = ApiSalesOrderStatus.completed;
        salesOrder.receivedDate = new Date();
        salesOrder.exchangeRate = 1.000000;
        salesOrder.discountRate = 0.0000;
        salesOrder.taxRate = 0.1500;
        salesOrder.xeroTaxCode = "OUTPUT2";
        salesOrder.subTotal = 275.000;
        salesOrder.taxTotal = 41.250;
        salesOrder.total = 316.250;
        salesOrder.totalVolume = 25.00;
        salesOrder.totalWeight = 25.00;
        salesOrder.baseCurrencySubTotal = 275.000;
        salesOrder.baseCurrencyTaxTotal = 0.000;
        salesOrder.baseCurrencyTotal = 275.000;
        salesOrder.paymentDueDate = new Date();
        salesOrder.createdBy = "acme@unleashedsoftware.com";
        createCustomer(salesOrder);
        createWarehouse(salesOrder);
        createCurrency(salesOrder);
        createTax(salesOrder);
        createSalesOrderLines(salesOrder);
        return salesOrder;
    }

    private void createCustomer(ApiSalesOrder salesOrder) {
        salesOrder.customer = new ApiCustomer();
        salesOrder.customer.id = customerId;
        salesOrder.customer.lastModifiedOn = new Date();
        salesOrder.customer.code = customerCode;
        salesOrder.customer.name = "DummyCustomer";
    }

    private void createWarehouse(ApiSalesOrder salesOrder) {
        salesOrder.warehouse = new ApiWarehouse();
        salesOrder.warehouse.id = warehouseId;
        salesOrder.warehouse.lastModifiedOn = new Date();
        salesOrder.warehouse.code = warehouseCode;
        salesOrder.warehouse.name = "DummyWarehouse";
        salesOrder.warehouse.isDefault = true;
    }

    private void createCurrency(ApiSalesOrder salesOrder) {
        salesOrder.currency = new ApiCurrency();
        salesOrder.currency.id = currencyId;
        salesOrder.currency.lastModifiedOn = new Date();
        salesOrder.currency.code = currencyCode;
        salesOrder.currency.description = "DummyDescription";
    }

    private void createTax(ApiSalesOrder salesOrder) {
        salesOrder.tax = new ApiTax();
        salesOrder.tax.lastModifiedOn = new Date();
        salesOrder.tax.id = taxId;
        salesOrder.tax.code = taxCode;
        salesOrder.tax.description = "DummyTax";
        salesOrder.tax.taxRate = 0.1500;
    }

    private void createSalesOrderLines(ApiSalesOrder salesOrder) {
        salesOrder.salesOrderLines = new ArrayList<ApiSalesOrderLine>();
        salesOrder.salesOrderLines.add(createSalesOrderLine1());
        salesOrder.salesOrderLines.add(createSalesOrderLine2());
    }

    private ApiSalesOrderLine createSalesOrderLine1() {
        ApiSalesOrderLine salesOrderLine1 = new ApiSalesOrderLine();
        salesOrderLine1.id = salesOrderLine1Id;
        salesOrderLine1.lastModifiedOn = new Date();
        salesOrderLine1.lineNumber = 1;
        salesOrderLine1.dueDate = new Date();
        salesOrderLine1.orderQuantity = 10.0000;
        salesOrderLine1.unitPrice = 5.000;
        salesOrderLine1.discountRate = 0.00000;
        salesOrderLine1.lineTotal = 50.00000000;
        salesOrderLine1.volume = 10.000;
        salesOrderLine1.weight = 10.000;
        salesOrderLine1.unitCost = 1.00000000000000;
        salesOrderLine1.taxRate = 0.1500;
        salesOrderLine1.lineTax = 7.500;
        salesOrderLine1.xeroTaxCode = "OUTPUT2";
        salesOrderLine1.baseCurrencyUnitPrice = 5.000;
        salesOrderLine1.baseCurrencyLineTotal = 50.000;
        salesOrderLine1.baseCurrencyLineTax = 7.500;
        createProduct1(salesOrderLine1);
        return salesOrderLine1;
    }

    private void createProduct1(ApiSalesOrderLine salesOrderLine1) {
        salesOrderLine1.product = new ApiProduct();
        salesOrderLine1.product.id = product1Id;
        salesOrderLine1.product.code = product1Code;
        salesOrderLine1.product.description = "DummyDescription";
    }

    private ApiSalesOrderLine createSalesOrderLine2() {
        ApiSalesOrderLine salesOrderLine2 = new ApiSalesOrderLine();
        salesOrderLine2.id = salesOrderLine2Id;
        salesOrderLine2.lastModifiedOn = new Date();
        salesOrderLine2.lineNumber = 2;
        salesOrderLine2.dueDate = new Date();
        salesOrderLine2.orderQuantity = 15.0000;
        salesOrderLine2.unitPrice = 15.000;
        salesOrderLine2.discountRate = 0.00000;
        salesOrderLine2.lineTotal = 225.00000000;
        salesOrderLine2.volume = 15.000;
        salesOrderLine2.weight = 15.000;
        salesOrderLine2.unitCost = 9.50000000000000;
        salesOrderLine2.taxRate = 0.1500;
        salesOrderLine2.lineTax = 33.750;
        salesOrderLine2.xeroTaxCode = "OUTPUT2";
        salesOrderLine2.baseCurrencyUnitPrice = 15.000;
        salesOrderLine2.baseCurrencyLineTotal = 225.000;
        salesOrderLine2.baseCurrencyLineTax = 33.750;
        createProduct2(salesOrderLine2);
        return salesOrderLine2;
    }

    private void createProduct2(ApiSalesOrderLine salesOrderLine2) {
        salesOrderLine2.product = new ApiProduct();
        salesOrderLine2.product.id = product2Id;
        salesOrderLine2.product.code = product2Code;
        salesOrderLine2.product.description = "DummyDescription";
    }
}